from typing import override
from h5py import File, Group
from backscattering_experimental_data import (
    Section,
    Measurement,
    MetadataRoot as Metadata,
)


class Root(Section):
    """
    Root object containing multiple measurement. It also can be called a
    package.
    """

    def __init__(
        self, metadata: Metadata, measurements: list[Measurement]
    ):
        """
        Parameters
        ----------
        metadata : bed.MetadataRoot
            Metadata associated to the data
        measurements : list[bed.Measurement]
            Measurements associated to the package
        """
        self.metadata = metadata
        self.measurements = measurements

    @override
    @staticmethod
    def from_hdf5(store: File | Group) -> "Root":
        measurements_group = store.get("measurements")
        if not isinstance(measurements_group, Group):
            raise ValueError(
                "measurements group should exists, and not be {}".format(
                    type(measurements_group)
                )
            )
        measurements = measurements_group.values()
        return Root(
            Metadata.from_hdf5(store),
            [
                Measurement.from_hdf5(measurement)
                for measurement in measurements
            ],
        )

    @override
    def to_hdf5(self, store: File | Group) -> Group | File:
        self.metadata.to_hdf5(store)
        for measurement in self.measurements:
            measurement.from_hdf5(
                store.require_group("measurements").create_group(
                    measurement.metadata.name,
                )
            )
        return store
